﻿namespace TabsStudioSQLComment
{
    public class SQLComment : TabsStudioExt.ITabsStudioAddin, TabsStudioExt.IConfigurable
    {
        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte)
        {
            engine = engine_;
            engine.TabCreated += OnTabCreated;

            storage = new OptionsStorage(engine.GetAddinSettingsDirectory());
            options = storage.Load();
        }

        public void OnDisconnection()
        {
            engine.TabCreated -= OnTabCreated;
        }

        public void Configure(System.Windows.Window parent)
        {
            OptionsDialog dialog = new OptionsDialog(options);
            dialog.Owner = parent;
            dialog.WindowStartupLocation = System.Windows.WindowStartupLocation.CenterOwner;
            if (dialog.ShowDialog().GetValueOrDefault())
                storage.Save(options);
        }

        private void OnTabCreated(object sender, TabsStudioExt.TabEventArgs e)
        {
            if (!IsPoorlyNamedTab(e.Tab))
                return;
            
            try
            {
                System.Text.RegularExpressions.Regex regex =
                    new System.Text.RegularExpressions.Regex(options.commentRegex, 
                        System.Text.RegularExpressions.RegexOptions.Multiline);
                System.Text.RegularExpressions.Match match = 
		            regex.Match(GetDocumentText(e.Tab.TabExtensions[0].Window.Document));

                string comment = "";
	            foreach(System.Text.RegularExpressions.Capture c in match.Groups["M"].Captures)
            		comment += c.Value;
                comment = comment.Replace("\r", "").Replace("\n", "");
                
                if (comment.Length > 0)
                    AddComment(e.Tab, comment);
            }
            catch (System.Exception)
            {
            }
        }

        private bool IsPoorlyNamedTab(TabsStudioExt.Tab tab)
        {
            System.Text.RegularExpressions.Regex regex =
                new System.Text.RegularExpressions.Regex(@"^SQLQuery\d+\.sql");
            return regex.IsMatch(tab.TitleManager.UntransformedTitle);
        }

        // throws System.Exception
        private string GetDocumentText(EnvDTE.Document doc)
        {
            EnvDTE.TextDocument textDoc = doc.Object("TextDocument") as EnvDTE.TextDocument;
            if (textDoc == null)
                return "";
            return textDoc.StartPoint.CreateEditPoint().GetText(textDoc.EndPoint);
        }

        private void AddComment(TabsStudioExt.Tab tab, string comment)
        {
            tab.TitleManager.Transforms.Add(
                new TabsStudioExt.TitleTransform("^.*$", "$_ " + comment, "[SQLComment]"));
            tab.TitleManager.ApplyTransforms();
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private OptionsStorage storage;
        private Options options;
    }
}
